'''
Author: bo-qian bqian@shu.edu.cn
Date: 2025-06-26 15:44:56
LastEditors: bo-qian bqian@shu.edu.cn
LastEditTime: 2025-06-27 13:13:48
FilePath: /boviz/tests/__init__.py
Description: This module initializes the test suite for boviz, ensuring all tests are discoverable and executable.
Copyright (c) 2025 by Bo Qian, All Rights Reserved. 
'''

# This file marks the tests directory as a Python package.
# Required in some environments for test discovery.