'''
Author: bo-qian bqian@shu.edu.cn
Date: 2025-07-01 15:56:53
LastEditors: bo-qian bqian@shu.edu.cn
LastEditTime: 2025-07-01 15:57:22
FilePath: /boviz/src/__main__.py
Description: This is the main entry point for the boviz CLI.
Copyright (c) 2025 by Bo Qian, All Rights Reserved. 
'''
from boviz.cli import main

if __name__ == '__main__':
    main()